<?php
/**
 *
 * HTML on posts. An extension for the phpBB Forum Software package.
 *
 * @copyright (c) 2022
 * @license GNU General Public License, version 2 (GPL-2.0)
 *
 */

namespace w3all\htmlposts\event;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class main_listener implements EventSubscriberInterface
{
  public static function getSubscribedEvents()
  {
    return [
      'core.viewtopic_modify_post_data' => 'viewtopic_modify_post_data',
      'core.posting_modify_submit_post_before' => 'posting_modify_submit_post_before',
    ];
  }

  protected $user;

  public function __construct(\phpbb\user $user)
  {
    $this->user = $user;
    $this->uids_html_allowed_ary = array(); // if you want to add HTML allowed user's IDS, do it in this way: array(48,625,34,765);
    $this->htmlposts_placeholder = '[HTMLMARKUP]';
  }

    public function posting_modify_submit_post_before($e)
  {
    // If the user that edit the post do not belong to the HTML array of usersIDS allowed to post html
    // and the author is an user allowed to post Html
    // then we do not allow to save the edited post to this user, even if he is a moderator

    if( substr($e['data']['message'], 3, 12) == $this->htmlposts_placeholder && !in_array($this->user->data['user_id'], $this->uids_html_allowed_ary) && in_array($e['data']['poster_id'], $this->uids_html_allowed_ary) )
    {
      $message = 'You cannot edit an HTML post.';
      trigger_error($message); // the user is not allowed to edit an HTML. Stop here.
      // the follow become totally not useful (so do not ask me why i leave it here, i really do not know)
      $e_data = $e['data'];
      $e_data['message'] = trim($e_data['message']);
      $e_data['message'] = str_replace($this->htmlposts_placeholder, "", $e_data['message']);
      $e['data'] = $e_data;
      unset($e_data);
     }
   }

  public function viewtopic_modify_post_data($e)
  {

    $e_rowset = $e['rowset'];
    foreach($e_rowset as $p => $pp){
        // ** only if the case going to execute this. Is this a post that should be parsed as HTML?
        // Then it should start with something like '<t>[HTMLMARKUP]' and if not, we skip all
     if ( substr($pp['post_text'], 3, 12) == $this->htmlposts_placeholder )
     {
        // ** ONLY if the post's author UID exist into the uids_html_allowed_ary we allow to parse the HTML on this post
       if( $pp['user_id'] > 1 && in_array($pp['user_id'], $this->uids_html_allowed_ary) ) // only posts which authors uids are allowed can follow here. Add more uids into array above
       {
          // removing some unwanted char that may occur
           $e_rowset[$p]['post_text'] = preg_replace('~[[:cntrl:]]~', '', $e_rowset[$p]['post_text']);
           // doing str_replace will not work
           // $e_rowset[$p]['post_text'] = str_replace($this->htmlposts_placeholder, "", $pp['post_text']);
           // lead to a wrong html parsing for tags attributes, so use substr to remove the (starting) [HTMLMARKUP] placeholder from text, then re-build the text with his starting tag
           // here we go
            $stag = substr($e_rowset[$p]['post_text'],0,3);
            $cpstr = substr($e_rowset[$p]['post_text'],15);
            $e_rowset[$p]['post_text'] = $stag . $cpstr;
          // strip BBCode/smilies (or the parsing of the post will make fall phpBB into an error)
           strip_bbcode($e_rowset[$p]['post_text']);
          // clean up unwanted br tags between (others tags), if there are
           $e_rowsetpost_text = preg_replace('~\<br ?\/?\>~umsi', '', $e_rowset[$p]['post_text']);
          // decode the post text
           $e_rowset[$p]['post_text'] = html_entity_decode($e_rowsetpost_text, ENT_QUOTES | ENT_SUBSTITUTE, 'UTF-8');
        }
       }
    }

   $e['rowset'] = $e_rowset;
   unset($e_rowset);
  }

}
